""" A test class used to test adapters. """


# Enthought library imports.
from traits.api import Interface


class IFoo(Interface):
    pass

#### EOF ######################################################################
