""" The exception raised when an unknown extension point is referenced. """


class UnknownExtensionPoint(Exception):
    """ The exception raised when an unknown extension point is referenced. """


#### EOF ######################################################################
